/* nhd_socket.c */
/* Norbert Doerry
   Novmeber 2018

*/

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "nhd_socket.h"

int sok;
struct sockaddr_in myaddress;
struct sockaddr_in theiraddress;

void nhd_init_socket(void)
{
 int bnd;
 int opt;

/* set up socket */

 sok = socket(AF_INET, SOCK_DGRAM, 0);
 
 if (sok == -1)
  {
   perror(" *** ERROR socket");
   exit(EXIT_FAILURE);
  }


 /* make the socket not blocking */

 if (fcntl(sok, F_SETFL, O_NONBLOCK) ==-1)
  {
   perror(" *** ERROR in fctl() SO_REUSEADDR");
   close(sok);
   exit(EXIT_FAILURE);
  }

 /* enable the socket to send broadcast messages */

 if (NHD_TYPE == 0)
  {
   opt = 1;
   if (setsockopt(sok,SOL_SOCKET,SO_BROADCAST,&opt,sizeof(opt)) == -1)
    {
     perror(" *** ERROR in setsockopt() SO_BROADCAST");
     close(sok);
     exit(EXIT_FAILURE);
    }

  }

 /* bind it to a port */

  myaddress.sin_family = AF_INET;

  if (NHD_TYPE)
   myaddress.sin_port = htons(NHD_PORT);  /* use specified port - server */
  else
   myaddress.sin_port = 0;  /* use any open port - client*/

  myaddress.sin_addr.s_addr = htonl(INADDR_ANY);  /* use the machines IP address */ 
  memset(&(myaddress.sin_zero), '\0', 8); // zero the rest of the struct

  bnd = bind(sok,(struct sockaddr *)&myaddress, sizeof(struct sockaddr));

 if (bnd == -1)
  {
   perror(" *** ERROR bind");
   close(sok);
   exit(EXIT_FAILURE);
  }

 theiraddress.sin_family = AF_INET;
 theiraddress.sin_port = htons(NHD_PORT);  // this will be overwritten if a server on first read

 if (inet_aton(NHD_SUBNET,&(theiraddress.sin_addr)) == 0)
  {
   perror(" *** ERROR inet_aton");
   close(sok);
   exit(EXIT_FAILURE);
  }

 memset(&(theiraddress.sin_zero), '\0', 8); // zero the rest of the struct

}

/* returns the number of characters received (0 if none) (-1 if error)*/

int nhd_socket_getbuf(char *buf)
{
   int fromlen;
   int rcf;

   if (buf == (char *) NULL)
    {
     fprintf(stderr," ***ERROR: NULL pointer in nhd_socket_getbuf\n");
     exit(EXIT_FAILURE);
    }
  
   fromlen = sizeof(struct sockaddr);

   rcf = recvfrom(sok,buf,NHD_BUF_LEN-1, 0, (struct sockaddr *) &theiraddress, &fromlen);

   if (rcf > 0 && rcf < NHD_BUF_LEN)
    {
     buf[rcf] = '\0';
     return rcf;
    }

   if (rcf == -1 && errno != EWOULDBLOCK && errno != EAGAIN)
    {
     perror(" *** ERROR recvfrom()");
     return -1;
    }

   else if (rcf == 0)
    {
     fprintf(stderr," *** ERROR: Remote Host closed connection\n");
     return -1;
    }
   else if (rcf != -1)
    {
     fprintf(stderr," *** ERROR: Unexpected rcf (%d)\n",rcf);
     close(sok);
     exit(EXIT_FAILURE);
    }


 return 0;

}


int nhd_socket_sendbuf(char *buf)
{
  int snd;

   if (buf == (char *) NULL)
    {
     fprintf(stderr," ***ERROR: NULL pointer in nhd_socket_getbuf\n");
     exit(EXIT_FAILURE);
    }
 
    snd = sendto(sok,buf,strlen(buf),0,(struct sockaddr *)&theiraddress,sizeof(struct sockaddr));

    if (snd == -1)
     {
      perror(" *** ERROR sendto");
      return snd;
     }

  return snd;
}


void nhd_socket_close(void)
{
 close(sok);
}

 